@@ -16,7 +16,7 @@ def _canonicalize(arguments, nan_as_missing):
 
 def _canonicalizeAll(arguments_df, nan_as_missing):
 	if nan_as_missing:
-		arguments_df = arguments_df.replace({numpy.NaN: None})
+		arguments_df = arguments_df.replace({numpy.nan: None})
 	return arguments_df
 
 class JavaBackend(ABC):
@@ -162,14 +162,17 @@ class Evaluator(JavaObject):
 
 	def evaluateAll(self, arguments_df, nan_as_missing = True):
 		arguments_df = _canonicalizeAll(arguments_df, nan_as_missing = nan_as_missing)
-		argument_records = arguments_df.to_dict(orient = "records")
-		argument_records = self.backend.dumps(argument_records)
+		arguments_dict = {
+			"columns" : arguments_df.columns.tolist(),
+			"data" : arguments_df.values.tolist()
+		}
+		arguments = self.backend.dumps(arguments_dict)
 		try:
-			result_records = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluateAll", self.javaEvaluator, argument_records)
+			results = self.backend.staticInvoke("org.jpmml.evaluator.python.PythonUtil", "evaluateAll", self.javaEvaluator, arguments)
 		except Exception as e:
 			raise self.backend.toJavaError(e)
-		result_records = self.backend.loads(result_records)
-		results_df = DataFrame.from_records(result_records)
+		results_dict = self.backend.loads(results)
+		results_df = DataFrame(data = results_dict["data"], index = (arguments_df.index.copy() if (len(arguments_dict["data"]) == len(results_dict["data"])) else None), columns = results_dict["columns"])
 		if hasattr(self, "dropColumns"):
 			for dropColumn in self.dropColumns:
 				results_df.drop(str(dropColumn), axis = 1, inplace = True)
@@ -229,6 +232,19 @@ class LoadingModelEvaluatorBuilder(BaseModelEvaluatorBuilder):
 		self.javaModelEvaluatorBuilder.load(file)
 		return self
 
+	def transform(self, javaPMMLTransformer):
+		self.javaModelEvaluatorBuilder.transform(javaPMMLTransformer)
+		return self
+
+	def transpile(self, path = None):
+		if path:
+			file = self.backend.newObject("java.io.File", path)
+			transpiler = self.backend.newObject("org.jpmml.transpiler.FileTranspiler", None, file)
+		else:
+			transpiler = self.backend.newObject("org.jpmml.transpiler.InMemoryTranspiler", None)
+		transformer = self.backend.newObject("org.jpmml.transpiler.TranspilerTransformer", transpiler)
+		return self.transform(transformer)
+
 def make_backend(alias):
 	if not isinstance(alias, str):
 		raise TypeError()
@@ -245,7 +261,7 @@ def make_backend(alias):
 		aliases = ["jpype", "pyjnius", "py4j"]
 		raise ValueError("Java backend alias {0} not in {1}".format(alias, aliases))
 
-def make_evaluator(path, backend = "jpype", lax = False, locatable = False, reporting = False):
+def make_evaluator(path, backend = "jpype", lax = False, locatable = False, reporting = False, transpile = False):
 	""" Builds an Evaluator based on a PMML file.
 
 	Parameters:
@@ -265,6 +281,9 @@ def make_evaluator(path, backend = "jpype", lax = False, locatable = False, repo
 
 	reporting: boolean
 		If True, activate the reporting Value API.
+
+	transpile: boolean or string
+		If not False, perform transpilation.
 	"""
 
 	if isinstance(backend, JavaBackend):
@@ -279,6 +298,8 @@ def make_evaluator(path, backend = "jpype", lax = False, locatable = False, repo
 		.loadFile(path)
 	if reporting:
 		evaluatorBuilder.setReportingValueFactoryFactory()
+	if transpile:
+		evaluatorBuilder.transpile(transpile if isinstance(transpile, str) else None)
 	return evaluatorBuilder.build()
 
 def _package_data_jars(package_data_dir):
